import torch

PITCH_MAX = 128
VELO = 128
LENGTH = 999
LENGTH_HALF = 999
BEGIN = 999
BEGIN_HALF = 999
ROOT = 99
START_SEQ_TOKEN = "<S_SEQ>"
END_SEQ_TOKEN = "<E_SEQ>"
PADDING_TOKEN = "<PAD>"

MODEL_NAME = "MORTM"

# 前回のトークンのID ＋　前回のトークンの使用個数
PADDING_BEGIN_ID = 0
SPECIAL_BEGIN_ID = PADDING_BEGIN_ID + 1
PITCH_BEGIN_ID = SPECIAL_BEGIN_ID + 2
VELOCITY_BEGIN_ID = PITCH_BEGIN_ID + 128
DURATION_BEGIN_ID = VELOCITY_BEGIN_ID + 128
START_BEGIN_ID = DURATION_BEGIN_ID + 100
SHIFT_BEGIN_ID = START_BEGIN_ID + 32


PITCH_GROUP = range(PITCH_BEGIN_ID, PITCH_BEGIN_ID + 128 + 1)
VELOCITY_GROUP = range(VELOCITY_BEGIN_ID, VELOCITY_BEGIN_ID + 128 + 1)
DURATION_GROUP = range(DURATION_BEGIN_ID, DURATION_BEGIN_ID + 100 + 1)
START_GROUP = range(START_BEGIN_ID, START_BEGIN_ID + 32 + 1)
SHIFT_GROUP = range(SHIFT_BEGIN_ID, SHIFT_BEGIN_ID + 4 + 1)

